require "showroom_api/version"
require 'uri'
require 'net/http'
require 'json'
#https://qiita.com/takeru7584/items/f4ba4c31551204279ed2

##
# This module represents showroom_api gem
module ShowroomApi
  ##
  # This class provides static methods for live rooms
  class Live
    # get the array of room categories and their available live rooms
    #
    # @param size [Integer]
    # @return [Array]
    #
    # @example
    #   ::ShowroomLive::Api::Live.onlives
    def self.onlives(size=nil)
      data=send_request("onlives",nil)[:onlives]
      cnt=ShowroomApi.to_integer(size)
      if !cnt.nil? && cnt > 0
        return data[0..(cnt-1)]  
      end
      return data
    end
    
    # get the id of the first room in the list of onlives
    #
    # @param size [Integer]
    # @return [Array]
    #
    # @example
    #   ::ShowroomLive::Api::Live.room_ids
    def self.room_ids(size)
      arr=[]
      cnt=ShowroomApi.to_integer(size)
      return [] if cnt.nil? || cnt <= 0
      rooms=onlives(1)
      rooms[0][:lives].each_with_index do |room,i|
        break if i>=cnt 
        arr<<room[:room_id]
      end
      #$stderr.puts arr
      return arr
    end
    
    # get the stream url of the first room in the list of onlives 
    #
    # @return [String]
    #
    # @example
    #   ::ShowroomLive::Api::Live.first_stream_url
    def self.first_stream_url
      return onlives(1)[0][:lives][0][:streaming_url_list][0][:url]
    end
    
    # get the info of the room
    #
    # @param room_id [String]
    # @return [Hash]
    #
    # @example
    #   ::ShowroomLive::Api::Live.live_info(room_id)
    def self.live_info(room_id)
      return send_request("live_info",room_id)
    end
    
    # get the list of room's streaming URLs
    #
    # @param room_id [String]
    # @return [Array]
    #
    # @example
    #   ::ShowroomLive::Api::Live.streaming_url(room_id)
    def self.streaming_url(room_id)
      send_request("streaming_url",room_id)[:streaming_url_list] || []
    end
    
    # get the live status of the room
    #
    # @param room_id [String]
    # @return [Hash]
    #
    # @example
    #   ::ShowroomLive::Api::Live.polling(room_id)
    def self.polling(room_id)
      return send_request("polling",room_id)
    end
    
    # age verification
    #
    # @param room_id [String]
    # @return [boolean]
    #
    # @example
    #   ::ShowroomLive::Api::Live.age_verification(room_id)
    def self.age_verification(room_id)
      return send_request("age_verification",room_id)[:is_passed]
    end
    
    # get logged in user's info
    #
    # @param room_id [String]
    # @return [Hash]
    #
    # @example
    #   ::ShowroomLive::Api::Live.current_user(room_id)
    def self.current_user(room_id)
      return send_request("current_user",room_id)
    end
    
    # telop
    #
    # @param room_id [String]
    # @return [Hash]
    #
    # @example
    #   ::ShowroomLive::Api::Live.telop(room_id)
    def self.telop(room_id)
      return send_request("telop",room_id)
    end
    
    # get room's survey result
    #
    # @param room_id [String]
    # @return [Hash]
    #
    # @example
    #   ::ShowroomLive::Api::Live.enquete_result(room_id)
    def self.enquete_result(room_id)
      return send_request("enquete_result",room_id)
    end
    
    # get room's ranking summary
    #
    # @param room_id [String]
    # @return [Array]
    #
    # @example
    #   ::ShowroomLive::Api::Live.summary_ranking(room_id)
    def self.summary_ranking(room_id)
      return send_request("summary_ranking",room_id)[:ranking] || []
    end
    
    # get room's comment log
    #
    # @param room_id [String]
    # @return [Array]
    #
    # @example
    #   ::ShowroomLive::Api::Live.comment_log(room_id)
    def self.comment_log(room_id)
      return send_request("comment_log",room_id)[:comment_log] || []
    end
    
    # get room's gift log
    #
    # @param room_id [String]
    # @return [Array]
    #
    # @example
    #   ::ShowroomLive::Api::Live.gift_log(room_id)
    def self.gift_log(room_id)
      return send_request("gift_log",room_id)[:gift_log] || []
    end
    
    # get room's available gift
    #
    # @param room_id [String]
    # @return [Hash]
    #
    # @example
    #   ::ShowroomLive::Api::Live.gift_list(room_id)
    def self.gift_list(room_id)
      return send_request("gift_list",room_id)
    end
    
    # get the list of gifts the room received
    #
    # @param room_id [String]
    # @return [Array]
    #
    # @example
    #   ::ShowroomLive::Api::Live.stage_gift_list(room_id)
    def self.stage_gift_list(room_id)
      return send_request("stage_gift_list",room_id)[:stage_gift_list] || []
    end
    
    # get the list of users in the room
    #
    # @param room_id [String]
    # @return [Array]
    #
    # @example
    #   ::ShowroomLive::Api::Live.stage_user_list(room_id)
    def self.stage_user_list(room_id)
      return send_request("stage_user_list",room_id)[:stage_user_list] || []
    end
    
    # get the count of live rooms
    #
    # @return [Integer]
    #
    # @example
    #   ::ShowroomLive::Api::Live.onlive_num
    def self.onlive_num
      return send_request("onlive_num",nil)[:num]
    end
    
    private
      # performs http request to showroom
      #
      # @param cmd [String]
      # @param room_id [String]
      # @return [Hash]
      def self.send_request(cmd,room_id=nil)
        url="https://www.showroom-live.com/api/live/"
        if room_id.nil? || room_id.to_s.empty?
          ShowroomApi.showroom_live_get_api_data("#{url}#{cmd}")
        else
          ShowroomApi.showroom_live_get_api_data("#{url}#{cmd}?room_id=#{room_id}")
        end
      end
  end
  
  ##
  # This class provides static methods for rooms
  class Room
    # get the info about event and support of the room
    #
    # @param room_id [String]
    # @return [Hash]
    #
    # @example
    #   ::ShowroomLive::Api::Room.event_and_support(room_id)
    def self.event_and_support(room_id)
      return send_request("event_and_support",room_id)
    end
    
    # get the profile info of the room
    #
    # @param room_id [String]
    # @return [Hash]
    #
    # @example
    #   ::ShowroomLive::Api::Room.profile(room_id)
    def self.profile(room_id)
      return send_request("profile",room_id)
    end
    
    # get the room's next live schedule
    #
    # @param room_id [String]
    # @return [Hash]
    #
    # @example
    #   ::ShowroomLive::Api::Room.next_live(room_id)
    def self.next_live(room_id)
      return send_request("next_live",room_id)
    end
    
    # get the room's available performance
    #
    # @param room_id [String]
    # @return [Array]
    #
    # @example
    #   ::ShowroomLive::Api::Room.settings(room_id)
    def self.settings(room_id)
      return send_request("settings",room_id)[:available_performance_type] || []
    end
    
    # get the status of the room
    #
    # @param room_id [String]
    # @return [Hash]
    #
    # @example
    #   ::ShowroomLive::Api::Room.status(room_id)
    def self.status(room_id)
      return send_request("status",room_id)
    end
    
    # get the name and screen_name of room's owner
    #
    # @param room_id [String]
    # @return [Hash]
    #
    # @example
    #   ::ShowroomLive::Api::Room.name(room_id)
    def self.name(room_id)
      url = "https://www.showroom-live.com/room/profile?room_id=#{room_id}"
      begin
        hash={}
        page = Nokogiri::HTML(open(url))
        hash[:screen_name] = page.xpath("//ul[contains(@class, 'room-profile-head-action-menu')]/li[1]/a/@href").first.value[1..-1]
        hash[:name] = page.xpath("//h2[contains(@class, 'room-profile-head-name')]/text()").first.text
        hash
      rescue
        {}
      end
    end
    
    private
      # performs http request to showroom
      #
      # @param cmd [String]
      # @param room_id [String]
      # @return [Hash]
      def self.send_request(cmd,room_id)
        ShowroomApi.showroom_live_get_api_data("https://www.showroom-live.com/api/room/#{cmd}?room_id=#{room_id}")
      end
  end
    
  ##
  # This class provides static methods for users
  class User
    # get the profile info of the user
    #
    # @param user_id [String]
    # @return [Hash]
    #
    # @example
    #   ::ShowroomLive::Api::User.send_request(user_id)
    def self.profile(user_id)
      return send_request("profile",user_id)
    end
    
    # get the detailed info of the logged-in user
    #
    # @return [Hash]
    #
    # @example
    #   ::ShowroomLive::Api::User.detail
    def self.detail
      return send_request("detail",nil)
    end
    
    # performs http request to showroom
    #
    # @param cmd [String]
    # @param user_id [String]
    # @return [Hash]
    private
      def self.send_request(cmd,user_id=nil)
        url="https://www.showroom-live.com/api/user/"
        if user_id.nil? || user_id.to_s.empty?
          ShowroomApi.showroom_live_get_api_data("#{url}#{cmd}")
        else
          ShowroomApi.showroom_live_get_api_data("#{url}#{cmd}?user_id=#{user_id}")
        end
      end
  end
  
  protected
    # performs http request to showroom
    #
    # @param url [String]
    # @return [Hash]
    def self.showroom_live_get_api_data(url)
      begin
        uri = URI(url)
        http = Net::HTTP.new(uri.host, uri.port)
        http.use_ssl = true
        http.verify_mode = OpenSSL::SSL::VERIFY_NONE
        request = Net::HTTP::Get.new(url)
        response = http.request(request)
        return JSON.parse(response.read_body,:symbolize_names => true)
      rescue
        return {}
      end
    end
    
    # convert string to valid integer
    #
    # @param num [String]
    # @return [Integer]
    def self.to_integer(num)
      return nil if num.nil?
      return num.to_i if num.to_i.to_s == num.to_s
      return nil
    end
end
